using System.Collections.Generic;
using com.agilefaqs.rft.domain;
using NUnit.Framework;
using Rhino.Mocks;

namespace com.agilefaqs.rft.suggestions.strategies
{
    [TestFixture]
    public class ChildSuggestionFactoryTest
    {
        private MockRepository m;
        private UserService userService;
        private SuggestionsUtil suggestionUtil;
        private List<ChildSuggestionStrategy> strategies;
        private List<ChildSuggestionStrategy> strategiesForSuggestionOne;

        [SetUp]
        public void SetUp()
        {
            m = new MockRepository();
            userService = m.StrictMock<UserService>();
            DomainNameService dns = m.Stub<DomainNameService>();
            suggestionUtil = m.StrictMock<SuggestionsUtil>(dns);
            strategies = new List<ChildSuggestionStrategy>();
            strategiesForSuggestionOne = new List<ChildSuggestionStrategy>();
        }

        [Test]
        public virtual void shouldGetAllTheStrategies()
        {
            SimpleTokenStrategy simpleTokenStrategy = new SimpleTokenStrategy(userService, suggestionUtil);

            YearBasedStrategy yearBasedStrategy = new YearBasedStrategy(userService, suggestionUtil);

            RandomNumberBasedStrategy randomNumberBasedStrategy =
                new RandomNumberBasedStrategy(userService, suggestionUtil, null);
            strategies.Add(simpleTokenStrategy);
            strategies.Add(new FirstNameAppendLastNameStrategy(userService, suggestionUtil));
            strategies.Add(new LastNameInitialStrategy(userService, suggestionUtil));
            strategies.Add(yearBasedStrategy);
            strategies.Add(new LastNameInitialYearBasedStrategy(userService, suggestionUtil));
            strategies.Add(randomNumberBasedStrategy);

            strategiesForSuggestionOne.Add(simpleTokenStrategy);
            strategiesForSuggestionOne.Add(yearBasedStrategy);
            strategiesForSuggestionOne.Add(randomNumberBasedStrategy);
            ChildSuggestionFactory factory = new ChildSuggestionFactory(userService, suggestionUtil, null);

            List<ChildSuggestionStrategy> allStrategies = factory.getStrategies();
            Assert.AreEqual(6, allStrategies.Count);


            List<ChildSuggestionStrategy> childStrategies = factory.getChildStrategiesForSuggestionOne();
            Assert.AreEqual(3, childStrategies.Count);
        }
    }
}