using System;
using com.agilefaqs.rft.domain;
using NUnit.Framework;
using Rhino.Mocks;

namespace com.agilefaqs.rft.suggestions.strategies
{
    [TestFixture]
    public class RandomNumberBasedStrategyTest
    {
        [SetUp]
        public void SetUp()
        {
            m = new MockRepository();
            DomainNameService dns = m.Stub<DomainNameService>();
            suggestionUtil = m.StrictMock<SuggestionsUtil>(dns);
            userService = m.StrictMock<UserService>();
            randomNumberGenerator = m.StrictMock<RandomNumberGenerator>();
            randomNumberBasedStrategy =
                new RandomNumberBasedStrategy(userService, suggestionUtil, randomNumberGenerator);
        }

        private MockRepository m;


        private const String COUNTRY_MONIKER = "indian";
        private const String FIRST_NAME = "Naresh";
        private const String RANDOM = "100000";
        private const String RANDOM1 = "1999";
        private const String RANDOM2 = "668678";
        private const String RANDOM3 = "8989";
        private const String RANDOM4 = "333";

        private readonly String Identity3 =
            String.Concat(String.Concat(String.Concat(String.Concat(FIRST_NAME, RANDOM3), "@"), COUNTRY_MONIKER), ".com");


        private readonly ChildStrategyParam childStrategyParam =
            new ChildStrategyParam(FIRST_NAME, COUNTRY_MONIKER, null);

        private readonly String Identity =
            String.Concat(String.Concat(String.Concat(String.Concat(FIRST_NAME, RANDOM), "@"), COUNTRY_MONIKER), ".com");

        private readonly String Identity1 =
            String.Concat(String.Concat(String.Concat(String.Concat(FIRST_NAME, RANDOM1), "@"), COUNTRY_MONIKER), ".com");

        private readonly String Identity2 =
            String.Concat(String.Concat(String.Concat(String.Concat(FIRST_NAME, RANDOM2), "@"), COUNTRY_MONIKER), ".com");

        private readonly String Identity4 =
            String.Concat(String.Concat(String.Concat(String.Concat(FIRST_NAME, RANDOM4), "@"), COUNTRY_MONIKER), ".com");

        private UserService userService;

        private RandomNumberGenerator randomNumberGenerator;

        private SuggestionsUtil suggestionUtil;


        private RandomNumberBasedStrategy randomNumberBasedStrategy;

        [Test]
        public virtual void shouldGenerateSuggestionBasedOnLastNameIntialAndYearStratey()
        {
            String token1 = childStrategyParam.Token1;

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(new IDTokens(token1, token2));
            Expect.Call(randomNumberGenerator.next()).Return(RANDOM);
            Expect.Call(suggestionUtil.appendTokensForId(String.Concat(token1, RANDOM), token2)).Return(Identity);
            Expect.Call(userService.isIdentityAvailable(Identity)).Return(true);
            stopRecordingAndStartReplay();
            String Id = randomNumberBasedStrategy.generateId(childStrategyParam);
            Assert.AreEqual(Id, Identity);
        }

        [Test]
        public virtual void shouldReturnNullIfIdIsNotAvailable()
        {
            String token1 = childStrategyParam.Token1;

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(null);
            stopRecordingAndStartReplay();
            String generatedId = randomNumberBasedStrategy.generateId(childStrategyParam);
            Assert.IsNull(generatedId);
        }

        [Test]
        public virtual void shouldReturnNullIfIdIfMoreThanFiveAttempts()
        {
            String token1 = childStrategyParam.Token1;

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(new IDTokens(token1, token2));
            Expect.Call(randomNumberGenerator.next()).Return(RANDOM);
            Expect.Call(randomNumberGenerator.next()).Return(RANDOM1);
            Expect.Call(randomNumberGenerator.next()).Return(RANDOM2);
            Expect.Call(randomNumberGenerator.next()).Return(RANDOM3);
            Expect.Call(randomNumberGenerator.next()).Return(RANDOM4);
            Expect.Call(suggestionUtil.appendTokensForId(String.Concat(token1, RANDOM), token2)).Return(Identity);
            Expect.Call(suggestionUtil.appendTokensForId(String.Concat(token1, RANDOM1), token2)).Return(Identity1);
            Expect.Call(suggestionUtil.appendTokensForId(String.Concat(token1, RANDOM2), token2)).Return(Identity2);
            Expect.Call(suggestionUtil.appendTokensForId(String.Concat(token1, RANDOM3), token2)).Return(Identity3);
            Expect.Call(suggestionUtil.appendTokensForId(String.Concat(token1, RANDOM4), token2)).Return(Identity4);
            Expect.Call(userService.isIdentityAvailable(Identity)).Return(false);
            Expect.Call(userService.isIdentityAvailable(Identity1)).Return(false);
            Expect.Call(userService.isIdentityAvailable(Identity2)).Return(false);
            Expect.Call(userService.isIdentityAvailable(Identity3)).Return(false);
            Expect.Call(userService.isIdentityAvailable(Identity4)).Return(false);
            stopRecordingAndStartReplay();
            String Id = randomNumberBasedStrategy.generateId(childStrategyParam);
            Assert.IsNull(Id);
        }

        [TearDown]
        public virtual void verifyMocks()
        {
            m.VerifyAll();
        }

        private void stopRecordingAndStartReplay()
        {
            m.ReplayAll();
        }
    }
}