using System;
using com.agilefaqs.rft.domain;
using NUnit.Framework;
using Rhino.Mocks;

namespace com.agilefaqs.rft.suggestions.strategies
{
    [TestFixture]
    public class YearBasedStrategyTest
    {
        [SetUp]
        public void SetUp()
        {
            m = new MockRepository();
            DomainNameService dns = m.Stub<DomainNameService>();
            suggestionUtil = m.StrictMock<SuggestionsUtil>(dns);
            userService = m.StrictMock<UserService>();
            yearBasedStrategy = new YearBasedStrategy(userService, suggestionUtil);
        }

        private MockRepository m;


        private const String COUNTRY_MONIKER = "indian";

        private static readonly String CURRENT_YEAR = DateTime.Now.Year.ToString();
        private const String FIRST_NAME = "Naresh";
        private const String LAST_NAME = "Jain";

        private readonly ChildStrategyParam childStrategyParam =
            new ChildStrategyParam(FIRST_NAME, COUNTRY_MONIKER, LAST_NAME);


        private readonly String Identity =
            String.Concat(String.Concat(String.Concat(FIRST_NAME, "@"), COUNTRY_MONIKER), ".com");

        private SuggestionsUtil suggestionUtil;

        private UserService userService;


        private YearBasedStrategy yearBasedStrategy;


        [Test]
        public virtual void shouldGenerateSuggestionBasedOnYearStratey()
        {
            String token1 = childStrategyParam.Token1 + (CURRENT_YEAR);

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(new IDTokens(token1, token2));
            Expect.Call(suggestionUtil.appendTokensForId(token1, token2)).Return(Identity);
            Expect.Call(userService.isIdentityAvailable(Identity)).Return(true);
            stopRecordingAndStartReplay();
            String Id = yearBasedStrategy.generateId(childStrategyParam);
            Assert.AreEqual(Id, Identity);
        }

        [Test]
        public virtual void shouldReturnNullIfIdIsNotAvailable()
        {
            String token1 = childStrategyParam.Token1 + (CURRENT_YEAR);

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(null);
            stopRecordingAndStartReplay();
            String generatedId = yearBasedStrategy.generateId(childStrategyParam);
            Assert.IsNull(generatedId);
        }

        [Test]
        public virtual void shouldReturnNullIfIdTokenReturnedBySuggestionUtilIsNullBasedOnYearStratey()
        {
            String token1 = childStrategyParam.Token1 + (CURRENT_YEAR);

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(new IDTokens(token1, token2));
            Expect.Call(suggestionUtil.appendTokensForId(token1, token2)).Return(Identity);
            Expect.Call(userService.isIdentityAvailable(Identity)).Return(false);
            stopRecordingAndStartReplay();
            String generatedId = yearBasedStrategy.generateId(childStrategyParam);
            Assert.IsNull(generatedId);
        }

        [TearDown]
        public virtual void verifyMocks()
        {
            m.VerifyAll();
        }

        private void stopRecordingAndStartReplay()
        {
            m.ReplayAll();
        }
    }
}